@@ -32,8 +32,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <string.h>
-#include <libgnome/gnome-config.h>
-#include <libgnome/gnome-url.h>
 
 #include "yelp-bookmarks.h"
 #include "yelp-utils.h"
@@ -921,8 +919,8 @@ page_request_cb (YelpDocument       *document,
 
 	window->priv->current_request = -1;
 	yelp_page_free ((YelpPage *) func_data);
-	gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
-
+	gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)),
+	                       NULL);
 	g_free (data);
 	break;
     case YELP_DOCUMENT_SIGNAL_TITLE:
@@ -936,7 +934,8 @@ page_request_cb (YelpDocument       *document,
 	window_error (window, (gchar *) yelp_error_get_title (error),
 		      (gchar *) yelp_error_get_message (error), FALSE);
 	yelp_error_free (error);
-	gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
+	gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)),
+			       NULL);
 	break;
     default:
 	g_assert_not_reached();
@@ -1140,7 +1139,7 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
     }
 
     if (doc) {
-        gchar *slash;
+        gchar *faux_frag_id, *slash;
 	gboolean need_hist = FALSE;
 	if (!frag_id)
 	  frag_id = g_strdup ("x-yelp-index");
@@ -1149,22 +1148,35 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 				       priv->current_type == YELP_RRN_TYPE_XHTML ||
 				       priv->current_type == YELP_RRN_TYPE_TEXT))
 	    need_hist = TRUE;
-	window_setup_window (window, type, real_uri, frag_id,
-			     (gchar *) uri, current_base, need_hist);
 
         /* FIXME: Super hacky.  We want the part before the slash for mallard IDs,
-           because right now we're outputting "#page_id/section_id".  We ought to
-           be scrolling to the section as well.  I happen to know that get_page is
-           just going to strdup the string, and we free frag_id itself immediately
-           after this, so this is safe.  It just sucks.
+           because right now we're outputting "#page_id/section_id".
          */
-        slash = strchr (frag_id, '/');
+        faux_frag_id = g_strdup (frag_id);
+        slash = strchr (faux_frag_id, '/');
         if (slash)
             *slash = '\0';
-	priv->current_request = yelp_document_get_page (doc,
-							frag_id,
-							(YelpDocumentFunc) page_request_cb,
-							(void *) window);
+        if (strlen (faux_frag_id) == 0) {
+            gchar *new_frag_id, *newslash;
+            if (slash)
+                slash = g_strdup (slash + 1);
+            g_free (faux_frag_id);
+            faux_frag_id = g_strdup (priv->current_frag);
+            newslash = strchr (faux_frag_id, '/');
+            if (newslash)
+                *newslash = '\0';
+            new_frag_id = g_strconcat (faux_frag_id, "/", slash, NULL);
+            g_free (frag_id);
+            g_free (slash);
+            frag_id = new_frag_id;
+        }
+	window_setup_window (window, type, real_uri, frag_id,
+			     (gchar *) uri, current_base, need_hist);
+        priv->current_request = yelp_document_get_page (doc,
+                                                        faux_frag_id,
+                                                        (YelpDocumentFunc) page_request_cb,
+                                                        (void *) window);
+        g_free (faux_frag_id);
 	priv->current_document = doc;
     }
 
@@ -1312,11 +1324,10 @@ window_populate (YelpWindow *window)
 				   priv->back_menu);
 
     action = gtk_action_group_get_action(priv->action_group, "GoBack");
-
-    gtk_action_connect_proxy (action, b_proxy);
+    gtk_activatable_set_related_action (GTK_ACTIVATABLE (b_proxy), action);
 
     action = gtk_action_group_get_action (priv->action_group, "GoForward");
-    gtk_action_connect_proxy (action, f_proxy);
+    gtk_activatable_set_related_action (GTK_ACTIVATABLE (f_proxy), action);
 
     gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (f_proxy),
 				   priv->forward_menu);
@@ -1482,7 +1493,7 @@ window_key_event_cb (GtkWidget *widget, GdkEventKey *event,
 {
     if ((window->priv->search_action &&
 	 gtk_entry_action_has_focus ((GtkEntryAction *) window->priv->search_action)) ||
-	GTK_WIDGET_HAS_FOCUS (window->priv->find_entry))
+	gtk_widget_has_focus (window->priv->find_entry))
 	return FALSE;
 
     if (event->keyval == GDK_slash) {
@@ -1753,7 +1764,8 @@ window_do_load_html (YelpWindow    *window,
         g_object_unref (stream);
 
     g_free (real_uri);
-    gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
+    gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)),
+                           NULL);
 
     return handled;
 }
@@ -1771,7 +1783,8 @@ window_set_loading (YelpWindow *window)
     priv = window->priv;
 
     cursor = gdk_cursor_new (GDK_WATCH);
-    gdk_window_set_cursor (GTK_WIDGET (window)->window, cursor);
+    gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)),
+                           cursor);
     gdk_cursor_unref (cursor);
 
     action = gtk_action_group_get_action (priv->action_group, "GoPrevious");
@@ -1836,14 +1849,21 @@ html_uri_selected_cb (YelpHtml  *html,
 		      gboolean   handled,
 		      gpointer   user_data)
 {
+    gchar *new_uri = uri;
     YelpWindow *window = YELP_WINDOW (user_data);
 
     debug_print (DB_FUNCTION, "entering\n");
     debug_print (DB_ARG, "  uri = \"%s\"\n", uri);
 
+    if (g_str_has_prefix (uri, "xref:"))
+        new_uri = g_strconcat (window->priv->base_uri, "#", uri + 5, NULL);
+
     if (!handled) {
-	yelp_window_load (window, uri);
+	yelp_window_load (window, new_uri);
     }
+
+    if (new_uri != uri)
+        g_free (new_uri);
 }
 
 static gboolean
@@ -2347,6 +2367,7 @@ window_enable_cursor_cb (GtkAction *action, YelpWindow *window)
 static void
 history_load_entry (YelpWindow *window, YelpHistoryEntry *entry)
 {
+    gchar *slash, *frag_id;
     g_return_if_fail (YELP_IS_WINDOW (window));
 
     if (entry->type == YELP_RRN_TYPE_HTML || entry->type == YELP_RRN_TYPE_XHTML || entry->type == YELP_RRN_TYPE_TEXT) {
@@ -2359,10 +2380,21 @@ history_load_entry (YelpWindow *window, YelpHistoryEntry *entry)
 	    g_free (window->priv->base_uri);
 	window->priv->base_uri = g_strdup (entry->base_uri);
 	window->priv->current_document = entry->doc;
+
+        /* FIXME: Super hacky.  We want the part before the slash for mallard IDs,
+           because right now we're outputting "#page_id/section_id".  We ought to
+           be scrolling to the section as well.
+         */
+        slash = strchr (entry->frag_id, '/');
+        if (slash)
+            frag_id = g_strndup (entry->frag_id, slash - entry->frag_id);
+        else
+            frag_id = g_strdup (entry->frag_id);
 	window->priv->current_request = yelp_document_get_page (entry->doc,
-							entry->frag_id,
-							(YelpDocumentFunc) page_request_cb,
-							(void *) window);
+                                                                frag_id,
+                                                                (YelpDocumentFunc) page_request_cb,
+                                                                (void *) window);
+        g_free (frag_id);
     }
 
 }
@@ -2556,7 +2588,7 @@ window_about_cb (GtkAction *action, YelpWindow *window)
 {
     const gchar *copyright =
 	"Copyright © 2001-2003 Mikael Hallendal\n"
-	"Copyright © 2003-2005 Shaun McCance\n"
+	"Copyright © 2003-2009 Shaun McCance\n"
 	"Copyright © 2005-2006 Don Scorgie\n"
 	"Copyright © 2005-2006 Brent Smith";
     const gchar *authors[] = {
@@ -2835,9 +2867,16 @@ window_write_html (YelpLoadData *data)
     gchar contents[BUFFER_SIZE];
 
     /* Use a silly fake URI to stop gecko doing silly things */
-    if (data->window->priv->current_frag)
-	uri = g_strdup_printf ("%s#%s", data->window->priv->base_uri,
-			       data->window->priv->current_frag);
+    if (data->window->priv->current_frag) {
+        /* FIXME: more of this terrible slash hack */
+        gchar *slash, *jump_id;
+        slash = strchr (data->window->priv->current_frag, '/');
+        if (slash)
+            jump_id = slash + 1;
+        else
+            jump_id = data->window->priv->current_frag;
+	uri = g_strdup_printf ("%s#%s", data->window->priv->base_uri, jump_id);
+    }
     else
 	uri = g_strdup (data->window->priv->base_uri);
 
